# Copyright (c) 2007-2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.


"""
pesto.session.filesessionmanager
--------------------------------

Store request sessions in flat files.

Usage::

    >>> from pesto.session import session_middleware
    >>> from pesto.session.filesessionmanager import FileSessionManager
    >>> def my_app(environ, start_response):
    ...     start_response('200 OK', [('Content-Type', 'text/html')])
    ...     yield "<html>Whoa nelly!</html>"
    ...
    >>> manager = FileSessionManager('./sessions')
    >>> app = session_middleware(manager)(my_app)

"""

__docformat__ = 'restructuredtext en'
__all__ = ['FileSessionManager']

import cPickle
import logging
import os
import time
import stat

from pesto.session.base import SessionManagerBase

class FileSessionManager(SessionManagerBase):
    """
    A File-backed session manager.

    Synopsis::

        >>> from pesto.session import session_middleware
        >>> from pesto.session.filesessionmanager import FileSessionManager
        >>> manager = FileSessionManager('/tmp/sessions')
        >>> def app(environ, start_response):
        ...     "WSGI application code here"
        ...
        >>> app = session_middleware(manager)(app)
        >>>

    """

    def __init__(self, directory):
        """
        directory
            Path to directory in which session files will be stored.
        """
        super(FileSessionManager, self).__init__()
        self.directory = os.path.join(directory, '_pesto_sessions')

    def acquire_lock(self, session_id=None):
        """
        Acquire lock for the storage
        """

    def release_lock(self, session_id=None):
        """
        Release lock for the storage
        """

    def get_path(self, session_id):
        """
        Return the path to the file where session data is stored.

        Synopsis::

            >>> from pesto.session.base import Session
            >>> fsm = FileSessionManager(directory='/tmp')
            >>> session = Session(fsm, 'abcdefgh', True)
            >>> fsm.get_path(session.session_id)
            '/tmp/_pesto_sessions/ab/abcdefgh'
        """

        return os.path.join(self.directory, session_id[:2], session_id)

    def store(self, session):
        """
        Store ``session`` to a file
        """
        path = self.get_path(session.session_id)
        try:
            os.makedirs(os.path.dirname(path))
        except OSError:
            # Path exists or cannot be created. The latter error will be
            # picked up later :)
            pass

        f = open(path, 'w')
        try:
            cPickle.dump(session.data, f)
        finally:
            f.close()

    def _get_session_data(self, session_id):
        path = self.get_path(session_id)

        try:
            f = open(path, 'r')
        except IOError:
            return None
        try:
            try:
                return cPickle.load(f)
            except (EOFError, IOError):
                logging.exception("Could not read session %r", session_id)
                return None
        finally:
            f.close()

    def update_access_time(self, session_id):
        """
        Update session access time, by calling ``os.utime`` on the session file.
        """
        try:
            os.utime(self.get_path(session_id), None)
        except OSError:
            pass

    def get_access_time(self, session_id):
        """
        Return the time the given session was last accessed. Note that this
        uses the underlying filesystem's atime attribute, so will not work on
        filesystems mounted with noatime
        """
        return os.stat(self.get_path(session_id)).st_atime

    def _remove(self, session_id):
        try:
            os.unlink(self.get_path(session_id))
        except OSError:
            logging.exception("Could not remove session file for %r", self.get_path(session_id))

    def _purge_candidates(self, olderthan=1800):
        remove_from = time.time() - olderthan
        for directory, dirs, files in os.walk(self.directory):
            for filename in files:
                if os.stat(os.path.join(directory, filename))[stat.ST_MTIME] < remove_from:
                    yield filename

    def __contains__(self, session_id):
        return os.path.exists(self.get_path(session_id))





# Copyright (c) 2007-2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

import re
from time import time, sleep
import shutil
from tempfile import mkdtemp

from nose.tools import assert_equal

import pesto
from pesto.response import Response
from pesto.core import to_wsgi
from pesto.testing import TestApp
from pesto.session.base import ID_LENGTH
from pesto.session.memorysessionmanager import MemorySessionManager
from pesto.session.filesessionmanager import FileSessionManager

def get_session_id(response):
    cookie = response.get_header('Set-Cookie')
    if cookie:
        return re.match('pesto_session=([\w\d]+);', cookie).group(1)
    return None

def test_cookie():

    sessionmanager = MemorySessionManager(cache_size=10)

    @to_wsgi
    def app(request):
        return Response(['ok'])

    response = TestApp(
        pesto.session_middleware(
            sessionmanager,
            auto_purge_every=0,
        )(app)
    ).get('/')


    # Check we got a session id back
    session_id = get_session_id(response)
    assert len(session_id) == ID_LENGTH


def test_persistence():

    @to_wsgi
    def app(request):
        request.session['counter'] = request.session.get('counter', -1) + 1
        return Response()

    sessionmanager = MemorySessionManager(cache_size=10)
    app = pesto.session_middleware(sessionmanager)(app)

    app = TestApp(app)
    response = app.get('/')

    # Check we get a session id back
    session_id = get_session_id(response)
    assert len(session_id) == ID_LENGTH

    # Now, send back the session id we've just got. Check that we get the same
    # session back each time and that the session counter is incrementing

    for ix in range(10):

        response = app.get(
            '/',
            HTTP_COOKIE="pesto_session=" + session_id
        )
        assert_equal(sessionmanager.load(session_id)['counter'], ix + 1) # Add one for initial run

def test_persistence_qs():

    @to_wsgi
    def app(request):
        request.session['counter'] = request.session.get('counter', -1) + 1
        return Response()

    sessionmanager = MemorySessionManager(cache_size=10)
    app = TestApp(pesto.session_middleware(sessionmanager, persist='querystring')(app))

    response = app.get('/')
    assert_equal(response.status, '302 Found')
    uri = response.get_header('Location')
    from urlparse import urlparse
    uri = urlparse(uri)
    assert 'pesto_session' in uri.query
    response = app.get(uri.path, QUERY_STRING=uri.query)

    # Check we get a session id back
    session_id = re.search('pesto_session=([\w\d]+)', uri.query).group(1)
    assert_equal(len(session_id), ID_LENGTH)

    # Now, send back the session id we've just got. Check that we get the same
    # session back each time and that the session counter is incrementing

    for ix in range(10):

        assert_equal(app.get(uri.path, QUERY_STRING=uri.query).status, '200 OK')
        assert_equal(sessionmanager.load(session_id)['counter'], ix + 1) # Add one for initial run


def test_is_new():

    sessionmanager = MemorySessionManager(cache_size=10)

    @to_wsgi
    def first_app(request):
        assert request.session.is_new
        return Response()

    first_app = pesto.session_middleware(sessionmanager)(first_app)

    @to_wsgi
    def second_app(request):
        assert not request.session.is_new
        return Response()

    second_app = pesto.session_middleware(sessionmanager)(second_app)

    response = TestApp(first_app).get('/')
    session_id = get_session_id(response)

    response = TestApp(second_app).get(
        '/', HTTP_COOKIE="pesto_session=" + session_id
    )



def test_expiry():

    sessionmanager = MemorySessionManager(cache_size=10)

    @to_wsgi
    def app(request):
        return Response(['ok'])

    app = pesto.session_middleware(
        sessionmanager,
        auto_purge_every=0.1,
        auto_purge_olderthan=0.2
    )(app)
    app = TestApp(app)


    response = app.get('/')

    session_id = get_session_id(response)
    response = app.get('/', HTTP_COOKIE="pesto_session=" + session_id)

    # Should not have had time to expire (yet) therefore we won't get a cookie
    # back - the original one is still valid.
    assert get_session_id(response) is None

    # Allow time for session to expire
    sleep(1)

    response = app.get('/', HTTP_COOKIE="pesto_session=" + session_id)

    # Check we got a new session id back
    assert get_session_id(response) != session_id


def test_cookie_once_only():
    """
    Make sure the session middleware sends back a cookie only when the session
    is created and not on subsequent requests.
    """
    sessionmanager = MemorySessionManager(cache_size=10)

    @to_wsgi
    def app(request):
        return Response(['hello world'])

    app = pesto.session_middleware(sessionmanager)(app)

    app = TestApp(app)
    response = app.get('/')
    assert response.get_header("Set-Cookie")
    session_id = get_session_id(response)

    response = app.get('/', HTTP_COOKIE="pesto_session=" + session_id)
    assert not response.get_header("Set-Cookie")


def test_cookie_path_is_set_from_script_name():
    sessionmanager = MemorySessionManager(cache_size=10)

    @to_wsgi
    def app(request):
        return Response(['hello world'])

    app = pesto.session_middleware(sessionmanager)(app)
    response = TestApp(app).get('/myapp/test', SCRIPT_NAME='/myapp')
    assert 'Path=/myapp' in response.get_header("Set-Cookie")

def test_cookie_path_can_be_overridden_via_configuration():

    sessionmanager = MemorySessionManager(cache_size=10)

    @to_wsgi
    def app(request):
        return Response(['hello world'])

    app = pesto.session_middleware(sessionmanager, cookie_path='/another-path')(app)
    response = TestApp(app).get('/myapp/test', SCRIPT_NAME='/myapp')
    assert 'Path=/myapp' not in response.get_header("Set-Cookie")
    assert 'Path=/another-path' in response.get_header("Set-Cookie")

def test_cookie_domain_can_be_overridden_via_configuration():

    @to_wsgi
    def app(request):
        return Response(['hello world'])

    app = pesto.session_middleware(MemorySessionManager())(app)
    response = TestApp(app).get('/myapp/test', HTTP_HOST='subdomain.example.com')
    assert 'Domain=' not in response.get_header("Set-Cookie")

    app = pesto.session_middleware(MemorySessionManager(), cookie_domain='.example.com')(app)
    response = TestApp(app).get('/myapp/test', HTTP_HOST='subdomain.example.com')
    assert 'Domain=.example.com' in response.get_header("Set-Cookie")

def test_access_time_is_set_on_session_creation_memory():

    manager = MemorySessionManager()
    app = pesto.session_middleware(manager)(Response([]))

    now = time()
    session_id = get_session_id(TestApp(app).get('/'))
    assert (manager.get_access_time(session_id) - now) < 100

def test_access_time_is_set_on_session_creation_filesystem():
    sessiondir = mkdtemp()
    try:
        manager = FileSessionManager(sessiondir)
        app = pesto.session_middleware(manager)(Response([]))

        now = time()
        session_id = get_session_id(TestApp(app).get('/'))
        assert (manager.get_access_time(session_id) - now) < 100
    finally:
        shutil.rmtree(sessiondir)




# Copyright (c) 2007-2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

import logging
from StringIO import StringIO
from urlparse import urlparse
from urllib import quote
import re
import time
from tempfile import NamedTemporaryFile

import pesto
from pesto.wsgiutils import make_query
from pesto.testing import TestApp, make_environ
from pesto.response import Response
from pesto.request import Request
from pesto.core import to_wsgi
from pesto.cookie import Cookie
from pesto.dispatch import dispatcher_app
from pesto.httputils import RequestParseError

from nose.tools import assert_equal, assert_true, assert_raises

def test_response_header():
    @to_wsgi
    def app(request):
        return Response(
            x_myheader=['Foo', 'Bar']
        )
    response = TestApp(app).get('/')
    assert_equal(response.get_header('X-Myheader'), 'Bar,Foo')

def test_add_response_header():
    @to_wsgi
    def app(request):
        res = Response(
            x_myheader='Foo'
        )
        return res.add_header('X-Myheader', 'Bar')
    response = TestApp(app).get('/')
    assert_equal(response.get_header('X-Myheader'), 'Bar,Foo')

def test_default_content_type_header():
    @to_wsgi
    def app(request):
        return Response()
    assert_equal(TestApp(app).get('/').get_header('Content-Type'), 'text/html; charset=UTF-8')

def test_set_content_type_header():
    @to_wsgi
    def app(request):
        return Response(content_type='text/rtf; charset=ISO-8859-1')
    assert_equal(TestApp(app).get('/').get_header('Content-Type'), 'text/rtf; charset=ISO-8859-1')

def test_redirect():
    @to_wsgi
    def app(request):
        return Response.redirect('/new_location', request)
    response = TestApp(app).get('/')
    assert 'http://localhost/new_location' in response.body
    assert_equal(response.get_header('Location'), 'http://localhost/new_location')
    assert_equal(response.status, '302 Found')

def test_not_found():
    @to_wsgi
    def app(request):
        return Response.not_found()
    assert 'not found' in TestApp(app).get('/').body.lower()

def test_method_not_allowed():
    @to_wsgi
    def app(request):
        return Response.method_not_allowed(valid_methods=('PUT', 'DELETE'))
    response = TestApp(app).get('/')
    assert 'not allowed' in response.body.lower()
    assert_equal(response.status, '405 Method Not Allowed')
    assert_equal(response.get_header('Allow'), 'PUT,DELETE')

def test_set_cookie():
    @to_wsgi
    def app(request):
        return Response(set_cookie=Cookie(name='fruit', value='banana'))
    assert_equal(
        TestApp(app).get().get_header('Set-Cookie'),
        'fruit=banana;Version=1'
    )

def test_method_not_allowed_dispatcher():
    import logging; logging.getLogger().setLevel(logging.DEBUG)
    dispatcher = dispatcher_app()

    @dispatcher.match('/blah', 'DELETE', 'PUT')
    def app(request):
        return Response.method_not_allowed(request)

    response = TestApp(dispatcher).get('/blah')
    assert 'not allowed' in response.body.lower()
    assert_equal(response.status, "405 Method Not Allowed")
    assert_equal(response.get_header('Allow'), 'PUT,DELETE')

def test_current_request():
    r1 = Request(make_environ())
    r2 = Request(make_environ())
    assert r1 is not pesto.currentrequest()
    assert r2 is pesto.currentrequest()

def test_current_request_threaded():

    import threading

    r1 = Request(make_environ())

    def thread_test():
        r2 = Request(make_environ())
        assert r2 is pesto.currentrequest()

    t = threading.Thread(target=thread_test)
    t.start()
    assert r1 is pesto.currentrequest()
    t.join()


def test_request_uri():

    request = Request(make_environ(SCRIPT_NAME='/foo', PATH_INFO='/bar', HTTP_HOST='localhost:80'))
    assert_equal(request.request_uri, "http://localhost/foo/bar")

    request = Request(make_environ(SCRIPT_NAME='/foo/bar', PATH_INFO='', HTTP_HOST='localhost:80'))
    assert_equal(request.request_uri, "http://localhost/foo/bar")

    request = Request(make_environ(SCRIPT_NAME='/', PATH_INFO='foo/bar', HTTP_HOST='localhost:80'))
    assert_equal(request.request_uri, "http://localhost/foo/bar")

def test_request_path():

    request = Request(make_environ(SCRIPT_NAME='/foo', PATH_INFO='/bar', HTTP_HOST='localhost:80'))
    assert_equal(request.request_path, "/foo/bar")

    request = Request(make_environ(SCRIPT_NAME='/foo/bar', PATH_INFO='', HTTP_HOST='localhost:80'))
    assert_equal(request.request_path, "/foo/bar")

    request = Request(make_environ(SCRIPT_NAME='/', PATH_INFO='foo/bar', HTTP_HOST='localhost:80'))
    assert_equal(request.request_path, "/foo/bar")

def test_form():

    request = Request(make_environ(QUERY_STRING='foo=bar'))
    assert_equal(request.get('foo'), u'bar')
    assert_equal(request['foo'], u'bar')
    assert_equal(request.get('fuz'), None)

    request = Request(make_environ(QUERY_STRING='foo=bar&foo=baz'))
    assert_equal(request.get('foo'), u'bar')
    assert_equal(request['foo'], u'bar')
    assert_equal(request.getlist('foo'), [u'bar', u'baz'])

def test_remote_addr():
    request = Request(make_environ(REMOTE_ADDR="1.2.3.4", HTTP_X_FORWARDED_FOR="4.3.2.1"))
    assert request.remote_addr == "1.2.3.4"

def test_mock_post():
    @to_wsgi
    def app(request):
        return Response(content=[request.request_method, request.get('foo')])
    assert TestApp(app).post('/', data={'foo': 'bar'}).content == ['POST', 'bar']

def test_mock_get():
    @to_wsgi
    def app(request):
        return Response(content=[request.request_method, request.get('foo')])
    assert TestApp(app).get('/', data={'foo': 'baz'}).content == ['GET', 'baz']

def test_encoding():

    # Test UTF-8 decoding works (this should be the default)
    request = Request(make_environ(QUERY_STRING=make_query(foo=u'\xa0'.encode('utf8'))))
    assert_equal(request.get('foo'), u'\xa0')

    # Force a different encoding
    request = Request(make_environ(QUERY_STRING=make_query(foo=u'\xa0'.encode('utf7'))))
    request.charset = 'utf-7'
    assert_equal(request.get('foo'), u'\xa0')

    # Force an incorrect encoding
    request = Request(make_environ(QUERY_STRING=make_query(foo=u'\xa0'.encode('utf8'))))
    request.charset = 'utf-7'
    assert_raises(RequestParseError, request.get, 'foo')

def test_form_getters():

    alpha = u'\u03b1' # Greek alpha
    beta = u'\u03b2' # Greek beta
    gamma = u'\u03b3' # Greek gamma

    request = Request(make_environ(
        QUERY_STRING='foo=%s;foo=%s;bar=%s' % (
            quote(alpha.encode('utf-8')),
            quote(beta.encode('utf-8')),
            quote(gamma.encode('utf-8'))
        )
    ))
    assert request.get('foo') == request.form.get('foo') == alpha
    assert request.getlist('foo') == request.form.getlist('foo') == [alpha, beta]

    assert request.get('cheese') == request.form.get('cheese') == None
    assert request.getlist('cheese') == request.form.getlist('cheese') == []

    assert request.get('bar') == request.form.get('bar') == gamma
    assert request.getlist('bar') == request.form.getlist('bar') == [gamma]


def test_file_test():

    filename = 'test.txt'
    filedata = 'whoa nelly!\n'
    content_type = 'text/plain'
    boundary = '---------------------------1234'

    post_data = (
          '--%s\r\n' % boundary
        + 'Content-Disposition: form-data; name="uploaded_file"; filename="%s"\r\n' % filename
        + 'Content-Type: %s\r\n' % content_type
        + '\r\n'
        + filedata + '\r\n--' + boundary + '--\r\n'
    )
    request = Request(make_environ(
        wsgi_input=StringIO(post_data),
        REQUEST_METHOD='POST',
        CONTENT_TYPE='multipart/form-data; boundary=%s' % boundary,
        CONTENT_LENGTH=str(len(post_data)),
    ))
    assert_equal(request.files['uploaded_file'].filename, filename)
    assert_equal(request.files['uploaded_file'].headers['Content-Type'], content_type)

    ntf = NamedTemporaryFile()
    request.files['uploaded_file'].save(ntf)
    ntf.flush()

    f = open(ntf.name, 'r')
    testdata = f.read()
    f.close()
    ntf.close()
    assert_equal(testdata, filedata)

    ntf = NamedTemporaryFile()
    request.files['uploaded_file'].save(ntf.name)

    f = open(ntf.name, 'r')
    testdata = f.read()
    f.close()
    ntf.close()
    assert_equal(testdata, filedata)


def test_response_get_headers():

    response = Response(['whoa nelly!'], content_type='text/plain', x_test_header=["1", "2"])
    assert_equal(response.get_headers('content-type'), ['text/plain'])
    assert_equal(response.get_headers('x-test-header'), ['1', '2'])
    assert_equal(response.get_headers('X-Test-Header'), ['1', '2'])
    assert_equal(response.get_headers('x-does-not-exist'), [])

def test_response_get_headers():

    response = Response(['whoa nelly!'], content_type='text/plain', x_test_header=["1", "2"])
    assert_equal(response.get_header('content-type'), 'text/plain')
    assert_equal(response.get_header('x-test-header'), '1,2')
    assert_equal(response.get_header('X-Test-Header'), '1,2')
    assert_equal(response.get_header('x-does-not-exist', 'boo!'), 'boo!')

def test_from_wsgi():

    from pesto import to_wsgi
    def pesto_app(request):

        def content():
            yield "<html>"
            yield "Yop!"
            yield "</html>"

        return Response(content(), content_type='text/html', x_counter="1")

    wsgi_app = to_wsgi(pesto_app)
    def middleware(app):
        def middleware(environ, start_response):
            response = Response.from_wsgi(app, environ, start_response)
            return response.replace(x_counter='2')(environ, start_response)
        return middleware

    wsgi_app = middleware(wsgi_app)
    response = TestApp(wsgi_app).get('/')
    assert_equal(
        str(response),
        '200 OK\r\n'
        'Content-Type: text/html\r\n'
        'X-Counter: 2\r\n'
        '\r\n'
        '<html>Yop!</html>'
    )

def test_make_uri_quoting():
    request = Request(make_environ(HTTP_HOST='example.com', SCRIPT_NAME='/path with spaces/page one'))
    assert_equal(request.make_uri(), 'http://example.com/path%20with%20spaces/page%20one')
    assert_equal(request.make_uri(path='./page two'), 'http://example.com/path%20with%20spaces/page%20two')
    assert_equal(request.make_uri(path='/page three'), 'http://example.com/page%20three')


def test_onclose():

    closed = []
    @to_wsgi
    def app(request):
        def closeme():
            closed.append(1)
        def closeme2():
            closed.append(2)

        def content():
            yield "<html>"
            yield "Yop!"
            yield "</html>"

        return Response(content(), onclose=closeme).add_onclose(closeme2)

    def middleware(app):
        def middleware(environ, start_response):
            response = Response.from_wsgi(app, environ, start_response)
            return response.replace(x_counter='2')(environ, start_response)
        return middleware
    TestApp(app).get()
    assert_equal(closed, [1, 2])

def test_onclose_from_wsgi():
    """
    Check close events are propagated from an upstream WSGI app
    """
    closed = []

    class MyApp(object):

        def __init__(self):
            pass

        def __iter__(self):
            yield "blah"
            yield "blah"
            yield "blah"

        def __call__(self, environ, start_response):
            start_response('200 OK', [])
            return self

        def close(self):
            closed.append(1)

    def middleware(app):
        def middleware(environ, start_response):
            r = Response.from_wsgi(app, environ, start_response)
            return r.replace(content='blah')(environ, start_response)
        return middleware

    TestApp(middleware(MyApp())).get()
    assert_equal(closed, [1])


def test_delayed_start_response():

    # Most WSGI applications call start_response before returning the iterable.
    # However, an application may also call start_response in the first iteration.

    closed = []

    class wsgiapp(object):

        def __init__(self):
            self.iter_count = 0
            self._next = iter(['cat', 'sat', 'mat']).next

        def __call__(self, environ, start_response):
            self.environ = environ
            self.start_response = start_response
            return self

        def __iter__(self):
            return self

        def next(self):
            if self.iter_count == 0:
                self.start_response('200 hoopy', [])
            self.iter_count += 1
            return self._next()

        def close(self):
            closed.append(1)

    def middleware(app):
        def middleware(environ, start_response):
            r = Response.from_wsgi(app, environ, start_response)
            return r(environ, start_response)
        return middleware

    response = TestApp(middleware(wsgiapp())).get()
    assert_equal(response.status, '200 hoopy')
    assert_equal(closed, [1])





# Copyright (c) 2007-2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

"""
Web session management.
"""

__docformat__ = 'restructuredtext en'
__all__ = ['session_middleware']

import logging
import os
import random
import re
import threading
from time import sleep, time

from pesto.request import Request
from pesto.response import Response
from pesto.cookie import Cookie
from pesto.wsgiutils import ClosingIterator

def get_session_id_from_querystring(environ):
    """
    Return the session from the query string or None if no session can be read.
    """
    pattern = re.escape(environ['pesto.sessionmanager'].COOKIE_NAME) + '=([0-9a-z]{%d})' % ID_LENGTH
    try:
        return re.search(pattern, environ.get("QUERY_STRING", "")).group(1)
    except AttributeError:
        return None

def get_session_id_from_cookie(environ):
    """
    Return the session from a cookie or None if no session can be read
    """
    cookie = Request(environ).cookies.get(environ['pesto.sessionmanager'].COOKIE_NAME)
    if cookie and is_valid_id(cookie.value):
        return cookie.value
    return None


try:
    import hashlib
    ID_LENGTH = hashlib.sha256().digest_size * 2
    def generate_id():
        """Generate a unique session ID"""
        return hashlib.sha256(
              str(os.getpid())
            + str(time())
            + str(random.random())
        ).hexdigest()

except ImportError:
    import sha
    ID_LENGTH = 40
    def generate_id():
        """Generate a unique session ID"""
        return sha.new(
              str(os.getpid())
            + str(time())
            + str(random.random())
        ).hexdigest()


def is_valid_id(session_id, pattern=re.compile('^[a-f0-9]{%d}$' % ID_LENGTH)):
    """
    Return True if ``session_id`` is a well formed session id. This must be
    a hex string as produced by hashlib objects' ``hexdigest`` method.

    Synopsis::

        >>> is_valid_id('a' * ID_LENGTH)
        True
        >>> is_valid_id('z' * ID_LENGTH)
        False
        >>> is_valid_id('a' * (ID_LENGTH - 1))
        False
    """
    try:
        return pattern.match(session_id) is not None
    except TypeError:
        return False


class Session(object):
    """
    Session objects store information about the http sessions
    """


    # Indicates whether the session is newly created (ie within the current request)
    is_new = True

    def __init__(self, session_manager, session_id, is_new, data=None):
        """
        Create a new session object within the given session manager.
        """

        self.session_manager = session_manager
        self._changed = False
        self.session_id = session_id
        self.is_new = is_new
        self.data = {}

        if data is not None:
            self.data.update(data)

    def save_if_changed(self):
        """
        Save the session in the underlying storage mechanism if the session is
        new or if it has been changed since being loaded.

        Note that this will only detect changes to the session object itself.
        If you store a mutable object within the session and change that
        then you must explicity call ``request.session.save`` to ensure
        your change is saved.

        Return
            ``True`` if the session was saved, or ``False`` if it
            was not necessary to save the session.
        """
        if self._changed or self.is_new:
            self.save()
            self._changed = False
            self.is_new = False
            return True
        return False

    def save(self):
        """
        Saves the session to the underlying storage mechanism.
        """
        self.session_manager.store(self)

    def setdefault(self, key, value=None):
        self._changed = True
        return self.data.setdefault(key, value)

    def pop(self, key, default):
        self._changed = True
        return self.data.pop(key, default)

    def popitem(self):
        self._changed = True
        return self.data.popitem()

    def clear(self):
        self._changed = True
        return self.data.clear()

    def has_key(self, key):
        return self.data.has_key(key)

    def items(self):
        return self.data.items()

    def iteritems(self):
        return self.data.iteritems()

    def iterkeys(self):
        return self.data.iterkeys()

    def itervalues(self):
        return self.data.itervalues()

    def update(self, other, **kwargs):
        self._changed = True
        return self.data.update(other, **kwargs)

    def values(self):
        return self.data.values()

    def get(self, key, default=None):
        return self.data.get(key, default)

    def __getitem__(self, key):
        return self.data[key]

    def __iter__(self):
        return self.data.__iter__()

    def invalidate(self):
        """
        invalidate and remove this session from the sessionmanager
        """
        self.session_manager.remove(self.session_id)
        self.session_id = None

    def __setitem__(self, key, val):
        self._changed = True
        return self.data.__setitem__(key, val)

    def __delitem__(self, key):
        self._changed = True
        return self.data.__delitem__(key)

    def text(self):
        """
        Return a useful text representation of the session
        """
        import pprint
        return "<%s id=%s, is_new=%s\n%s\n>" % (
                self.__class__.__name__, self.session_id, self.is_new,
                pprint.pformat(self)
        )

class SessionManagerBase(object):
    """
    Manages Session objects using an ObjectStore to persist the
    sessions.
    """
    # Which version of the pickling protocol to select.
    PICKLE_PROTOCOL = -1

    # Key to use in HTTP cookies
    COOKIE_NAME = "pesto_session"

    def load(self, session_id):
        """
        Load a session object from this sessionmanager.

        Note that if ``session_id`` cannot be found in the underlying storage,
        a new session id will be created.
        """
        self.acquire_lock(session_id)
        try:
            data = self._get_session_data(session_id)
            if data is None:
                # Generate a fresh session with a new id
                session = Session(self, generate_id(), is_new=False, data=data)
            else:
                session = Session(self, session_id, is_new=False, data=data)
            self.update_access_time(session.session_id)
            return session
        finally:
            self.release_lock(session_id)

    def update_access_time(self, session_id):
        raise NotImplementedError

    def get_access_time(self, session_id):
        raise NotImplementedError

    def acquire_lock(self, session_id=None):
        """
        Acquire a lock for the given session_id.

        If session_id is none, then the whole storage should be locked.
        """
        raise NotImplementedError

    def release_lock(self, session_id=None):
        raise NotImplementedError

    def read_session(self, session_id):
        """
        Return a session object from the given ``session_id``. If
        ``session_id`` is None a new session will be generated.

        Synopsis::

            >>> from pesto.session.memorysessionmanager import MemorySessionManager
            >>> sm = MemorySessionManager()
            >>> sm.read_session(None) #doctest: +ELLIPSIS
            <pesto.session.base.Session object at 0x...>

        """
        if session_id is not None:
            session = self.load(session_id)
        else:
            session = Session(self, generate_id(), is_new=True)
        self.update_access_time(session.session_id)
        return session

    def __contains__(self, session_id):
        """
        Return true if the given session id exists in this sessionmanager
        """
        raise NotImplementedError

    def store(self, session):
        """
        Save the given session object in this sessionmanager.
        """
        self.acquire_lock(session.session_id)
        try:
            self._store(session)
        finally:
            self.release_lock(session.session_id)

    def _store(self, session):
        """
        Write session data to the underlying storage.
        Subclasses must implement this method
        """

    def remove(self, session_id):
        """
        Remove the specified session from the session manager.
        """
        self.acquire_lock(session_id)
        try:
            self._remove(session_id)
        finally:
            self.release_lock(session_id)

    def _remove(self, session_id):
        """
        Remove the specified session from the underlying storage.
        Subclasses must implement this method
        """
        raise NotImplementedError

    def _get_session_data(self, session_id):
        """
        Return a dict of the session data from the underlying storage, or
        ``None`` if the session does not exist.
        """
        raise NotImplementedError

    def close(self):
        """
        Close the persistent store cleanly.
        """
        self.acquire_lock()
        try:
            self._close()
        finally:
            self.release_lock()

    def _close(self):
        """
        Default implementation: do nothing
        """

    def purge(self, olderthan=1800):
        for session_id in self._purge_candidates(olderthan):
            self.remove(session_id)

    def _purge_candidates(self, olderthan=1800):
        """
        Return a list of session ids ready to be purged from the session
        manager.
        """
        raise NotImplementedError


class ThreadsafeSessionManagerBase(SessionManagerBase):
    """
    Base class for sessioning to run in a threaded environment.

    DOES NOT GUARANTEE PROCESS-LEVEL SAFETY!
    """

    def __init__(self):
        super(ThreadsafeSessionManagerBase, self).__init__()
        self._access_times = {}
        self._lock = threading.RLock()

    def _purge_candidates(self, olderthan=1800):
        """
        Purge all sessions older than ``olderthan`` seconds.
        """
        # Re-importing time fixes exception raised on interpreter shutdown
        from time import time
        expiry = time() - olderthan
        self.acquire_lock()
        try:
            return [
                id for id, access_time in self._access_times.iteritems() if access_time < expiry
            ]
        finally:
            self.release_lock()

    def acquire_lock(self, session_id=None):
        self._lock.acquire()

    def release_lock(self, session_id=None):
        self._lock.release()

    def update_access_time(self, session_id):
        self.acquire_lock()
        try:
            self._access_times[session_id] = time()
        finally:
            self.release_lock()

    def get_access_time(self, session_id):
        """
        Return the time the given session_id was last accessed
        """
        return self._access_times[session_id]

    def _remove(self, session_id):
        """
        Subclasses should call this implementation to ensure the access_times
        dictionary is kept up to date.
        """
        try:
            del self._access_times[session_id]
        except KeyError:
            logging.warn("tried to remove non-existant session id %r", session_id)

    def __contains__(self, session_id):
        return session_id in self._access_times

def start_thread_purger(sessionmanager, howoften=60, olderthan=1800, lock=threading.Lock()):
    """
    Start a thread to purge sessions older than ``olderthan`` seconds every
    ``howoften`` seconds.
    """

    def _purge():
        while True:
            sleep(howoften)
            sessionmanager.purge(olderthan)

    lock.acquire()
    try:
        if hasattr(sessionmanager, '_purge_thread'):
            # Don't start the thread twice
            return
        sessionmanager._purge_thread = threading.Thread(target=_purge)
        sessionmanager._purge_thread.setDaemon(True)
        sessionmanager._purge_thread.start()

    finally:
        lock.release()

def session_middleware(
    session_manager,
    auto_purge_every=60,
    auto_purge_olderthan=1800,
    persist='cookie',
    cookie_path=None,
    cookie_domain=None
):
    """
    WSGI middleware application for sessioning.

    Synopsis::

        >>> from pesto.session.memorysessionmanager import MemorySessionManager
        >>> def my_wsgi_app(environ, start_response):
        ...     session = environ['pesto.session']
        ... 
        >>> app = session_middleware(MemorySessionManager())(my_wsgi_app)
        >>> 

    session_manager
        An implementation of ``pesto.session.base.SessionManagerBase``

    auto_purge_every
        If non-zero, a separate thread will be launched which will purge
        expired sessions every ``auto_purge_every`` seconds. In a CGI
        environment (or equivalent, detected via, ``environ['wsgi.run_once']``)
        the session manager will be purged after every request.

    auto_purge_olderthan
        Auto purge sessions older than ``auto_purge_olderthan`` seconds.

    persist
        Either ``cookie`` or ``querystring``. If set to ``cookie`` then
        sessions will be automatically persisted via a session cookie.

        If ``querystring`` then the session-id will be read from the
        querystring. However it is up to the underlying application to ensure
        that the session-id is embedded into all links generated by the
        application.

    cookie_path
        The path to use when setting cookies. If ``None`` this will be taken
        from the ``SCRIPT_NAME`` variable.

    cookie_domain
        The domain to use when setting cookies. If ``None`` this will not be
        set and the browser will default to the domain used for the request.
    """

    get_session_id = {
        'cookie': get_session_id_from_cookie,
        'querystring': get_session_id_from_querystring,
    }[persist]

    def middleware(app):

        def sessionmanager_middleware(environ, start_response):

            if environ['wsgi.run_once'] and auto_purge_every > 0:
                session_manager.purge(auto_purge_olderthan)

            environ['pesto.sessionmanager'] = session_manager
            session = session_manager.read_session(get_session_id(environ))
            environ['pesto.session'] = session

            my_start_response = start_response

            if persist == 'cookie' and session.is_new:
                def my_start_response(status, headers, exc_info=None):
                    _cookie_path = cookie_path
                    if _cookie_path is None:
                        _cookie_path = environ.get('SCRIPT_NAME')
                    if not _cookie_path:
                        _cookie_path = '/'
                    cookie = Cookie(
                        session_manager.COOKIE_NAME,
                        session.session_id,
                        path=_cookie_path,
                        domain=cookie_domain,
                        http_only=True
                    )
                    return start_response(
                        status,
                        list(headers) + [("Set-Cookie", str(cookie))],
                        exc_info
                    )

            elif persist == 'querystring':
                request = Request(environ)
                if session.is_new and environ['REQUEST_METHOD'] == 'GET':
                    query_session_id = request.query.get(session_manager.COOKIE_NAME)
                    if query_session_id != session.session_id:
                        new_query = [
                            item for item in request.query.iterallitems()
                            if item[0] != session_manager.COOKIE_NAME
                        ]
                        new_query.append((session_manager.COOKIE_NAME, session.session_id))
                        return ClosingIterator(
                            Response.redirect(
                                request.make_uri(query=new_query)
                            )(environ, my_start_response),
                            session.save_if_changed
                        )

            return ClosingIterator(app(environ, my_start_response), session.save_if_changed)

        if auto_purge_every > 0:
            start_thread_purger(
                session_manager,
                howoften = auto_purge_every,
                olderthan = auto_purge_olderthan
            )


        return sessionmanager_middleware

    return middleware





# Copyright (c) 2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

"""
pesto.httputils
---------------

Utility functions to handle HTTP data
"""
import email.message
import email.parser
import re
from urllib import unquote_plus
from shutil import copyfileobj

from pesto.utils import ExpandableOutput, SizeLimitedInput, PutbackInput, DelimitedInput

KB = 1024
MB = 1024 * KB

class RequestParseError(Exception):
    """
    Error encountered while parsing the HTTP request
    """
    def response(self):
        """
        Return a ``pesto.response.Response`` object to represent this error condition
        """
        from pesto.response import Response
        return Response.bad_request().add_header('X-Pesto-Exception', repr(self))

class TooBig(RequestParseError):
    """
    Request body is too big
    """
    def response(self):
        """
        Return a ``pesto.response.Response`` object to represent this error condition
        """
        from pesto.response import Response
        return Response.request_entity_too_large()

class MissingContentLength(RequestParseError):
    """
    No ``Content-Length`` header given
    """
    def response(self):
        """
        Return a ``pesto.response.Response`` object to represent this error condition
        """
        from pesto.response import Response
        return Response.length_required()

def dequote(s):
    """
    Example usage::

        >>> dequote('foo')
        'foo'
        >>> dequote('"foo"')
        'foo'
    """
    if len(s) > 1 and s[0] == '"' == s[-1]:
        return s[1:-1]
    return s

def parse_header(header):
    """
    Given a header, return a tuple of ``(value, [(parameter_name, parameter_value)])``.

    Example usage::

        >>> parse_header("text/html; charset=UTF-8")
        ('text/html', {'charset': 'UTF-8'})
        >>> parse_header("multipart/form-data; boundary=---------------------------7d91772e200be")
        ('multipart/form-data', {'boundary': '---------------------------7d91772e200be'})
    """
    items = header.split(';')
    pairs = [
        (name, dequote(value))
             for name, value in (
                 item.lstrip().split('=', 1)
                    for item in items[1:]
            )
    ]
    return (items[0], dict(pairs))

def parse_querystring(
    data,
    charset=None,
    strict=False,
    keep_blank_values=True,
    pairsplitter=re.compile('[;&]').split
):
    """
    Return ``(key, value)`` pairs from the given querystring::

        >>> list(parse_querystring('green%20eggs=ham;me=sam+i+am'))
        [(u'green eggs', u'ham'), (u'me', u'sam i am')]

    charset
        Character encoding used to decode values. If not specified,
        ``pesto.DEFAULT_CHARSET`` will be used.

    keep_blank_values
        if True, keys without associated values will be returned
        as empty strings. if False, no key, value pair will be returned.

    strict
        if ``True``, a ``ValueError`` will be raised on parsing errors.
    """

    if charset is None:
        charset = DEFAULT_CHARSET

    for item in pairsplitter(data):
        if not item:
            continue
        try:
            key, value = item.split('=', 1)
        except ValueError:
            if strict:
                raise RequestParseError("bad query field: %r" % (item,))
            if not keep_blank_values:
                continue
            key, value = item, ''

        try:
            yield unquote_plus(key).decode(charset), unquote_plus(value).decode(charset)
        except UnicodeDecodeError:
            raise RequestParseError("Invalid character data: can't decode using %r" % (charset,))

def parse_post(environ, fp, default_charset=None, max_size=16*KB, max_multipart_size=2*MB):
    """
    Parse the contents of an HTTP POST request, which may be either
    application/x-www-form-urlencoded or multipart/form-data encoded.

    Returned items are either tuples of (name, value) for simple string values
    or (name, FileUpload) for uploaded files.

    max_multipart_size
        Maximum size of total data for a multipart form submission

    max_size
        The maximum size of data allowed to be read into memory.

        For a application/x-www-form-urlencoded submission, this is the maximum
        size of the entire data.

        For a multipart/form-data submission, this is the maximum size of any
        single non file-upload field.
    """
    content_type, content_type_params = parse_header(
        environ.get('CONTENT_TYPE', 'application/x-www-form-urlencoded')
    )

    if default_charset is None:
        default_charset = DEFAULT_CHARSET
    charset = content_type_params.get('charset', default_charset)

    try:
        content_length = int(environ['CONTENT_LENGTH'])
    except (TypeError, ValueError, KeyError):
        raise MissingContentLength()

    if content_type == 'application/x-www-form-urlencoded':

        if content_length > max_size:
            raise TooBig("Content Length exceeds permitted size")
        return parse_querystring(SizeLimitedInput(fp, content_length).read(), charset)

    else:
        if content_length > max_multipart_size:
            raise TooBig("Content Length exceeds permitted size")
        try:
            boundary = content_type_params['boundary']
        except KeyError:
            raise RequestParseError("No boundary given in multipart/form-data content-type")
        return parse_multipart(SizeLimitedInput(fp, content_length), boundary, charset, max_size)

class HTTPMessage(email.message.Message):
    """
    Represent HTTP request message headers
    """

CHUNK_SIZE = 8192

def parse_multipart(fp, boundary, default_charset, max_size):
    """
    Parse data encoded as ``multipart/form-data``. Generate tuples of::

        (<field-name>, <data>)

    Where ``data`` will be a string in the case of a regular input field, or a
    ``FileUpload`` instance if a file was uploaded.

    fp
        input stream from which to read data
    boundary
        multipart boundary string, as specified by the ``Content-Disposition`` header
    default_charset
        character set to use for encoding, if not specified by a content-type header.
        In practice web browsers don't supply a content-type header so this
        needs to contain a sensible value.
    max_size
        Maximum size in bytes for any non file upload part
    """

    boundary_size = len(boundary)
    if not boundary.startswith('--'):
        raise RequestParseError("Malformed boundary string: must start with '--' (rfc 2046)")
    if boundary_size > 72:
        raise RequestParseError("Malformed boundary string: must be no more than 70 characters, not counting the two leading hyphens (rfc 2046)")

    assert boundary_size + 2 < CHUNK_SIZE, "CHUNK_SIZE cannot be smaller than the boundary string"

    if fp.read(2) != '--':
        raise RequestParseError("Malformed POST data: expected two hypens")

    if fp.read(boundary_size) != boundary:
        raise RequestParseError("Malformed POST data (expected boundary)")

    if fp.read(2) != '\r\n':
        raise RequestParseError("Malformed POST data (expected CRLF)")

    fp = PutbackInput(fp)

    while True:
        headers, data = _read_multipart_field(fp, boundary)
        try:
            disposition_type, params = parse_header(headers['Content-Disposition'])
        except KeyError:
            raise RequestParseError("Missing Content-Disposition header")

        try:
            name = params['name']
        except KeyError:
            raise RequestParseError("Missing name parameter in Content-Disposition header")

        is_file_upload = 'Content-Type' in headers and 'filename' in params
        if is_file_upload:
            io = data._io
            io.seek(0)
            yield name, FileUpload(params['filename'], headers, io)

        else:
            charset = parse_header(headers.get('Content-Type', ''))[1].get('charset', default_charset)
            if data.tell() > max_size:
                raise TooBig("Data block exceeds maximum permitted size")
            try:
                data.seek(0)
                yield name, data.read().decode(charset)
            except UnicodeDecodeError:
                raise RequestParseError("Invalid character data: can't decode using %r" % (charset,))

        chunk = fp.read(2)
        if chunk == '--':
            if fp.peek(3) != '\r\n':
                raise RequestParseError("Expected terminating CRLF at end of stream")
            break

        if chunk != '\r\n':
            raise RequestParseError("Expected CRLF after boundary")

CONTENT_DISPOSITION_FORM_DATA = 'form-data'
CONTENT_DISPOSITION_FILE_UPLOAD = 'file-upload'

def _read_multipart_field(fp, boundary):
    """
    Read a single part from a multipart/form-data message and return a tuple of
    ``(headers, data)``. Stream ``fp`` must be positioned at the start of the
    header block for the field.

    Return a tuple of ('<headers>', '<data>')

    ``headers`` is an instance of ``email.message.Message``.

    ``data`` is an instance of ``ExpandableOutput``.

    Note that this currently cannot handle nested multipart sections.
    """
    data = ExpandableOutput()
    parser = email.parser.Parser(_class=HTTPMessage)
    headers = parser.parse(DelimitedInput(fp, '\r\n\r\n'), headersonly=True)
    fp = DelimitedInput(fp, '\r\n--' + boundary)
    # XXX: handle base64 encoding etc
    for chunk in iter(lambda: fp.read(CHUNK_SIZE), ''):
        data.write(chunk)
    data.flush()

    # Fallen off the end of the input without having read a complete field?
    if not fp.delimiter_found:
        raise RequestParseError("Incomplete data (expected boundary)")

    return headers, data


class FileUpload(object):

    """
    Represent a file uploaded in an HTTP form submission
    """

    def __init__(self, filename, headers, fileob):

        self.filename = filename
        self.headers = headers
        self.file = fileob

        # UNC/Windows path
        if self.filename[:2] == '\\\\' or self.filename[1:3] == ':\\':
            self.filename = self.filename[self.filename.rfind('\\')+1:]

    def save(self, fileob):
        """
        Save the upload to the file object or path ``fileob``
        """
        if isinstance(fileob, basestring):
            fileob = open(fileob, 'w')
            try:
                return self.save(fileob)
            finally:
                fileob.close()

        self.file.seek(0)
        copyfileobj(self.file, fileob)

# Imports at end to avoid circular dependencies
from pesto import DEFAULT_CHARSET



# Copyright (c) 2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

"""
Tests for pesto/utils.py
"""

from __future__ import with_statement

from cStringIO import StringIO, OutputType as cStringIO_OutputType

from nose.tools import assert_equal, assert_true

from pesto.utils import ExpandableOutput, PutbackInput, DelimitedInput


def test_expandableio():

    with ExpandableOutput(10) as eio:
        eio.write('abc')
        eio.seek(0)
        assert_equal(eio.read(), 'abc')
        assert_true(isinstance(eio._io, cStringIO_OutputType))
        eio.write('defghij')
        assert_true(isinstance(eio._io, cStringIO_OutputType))
        eio.write('k')
        assert_true(isinstance(eio._io, file))
        eio.seek(0)
        assert_equal(eio.read(), 'abcdefghijk')

def test_putbackio_nested_putbacks():
    s = StringIO('one,two,three')
    s.seek(0)
    p = PutbackInput(s)
    a = s.read(3)
    b = s.read(3)
    p.putback(b)
    p.putback(a)
    assert_equal(p.read(), 'one,two,three')

def test_putbackio_read_from_putback_short():
    """
    Test read for a length smaller than the putback buffer
    """
    s = StringIO('one,two,three')
    s.seek(0)
    p = PutbackInput(s)
    a = p.read(4)
    b = p.read(4)
    p.putback(b)
    p.putback(a)
    assert_equal(p.read(7), "one,two")

def test_putbackio_read_from_putback_and_stream_long():
    """
    Test read for a length longer than the putback buffer
    """
    s = StringIO('one,two,three')
    s.seek(0)
    p = PutbackInput(s)
    a = p.read(2)
    b = p.read(2)
    p.putback(b)
    p.putback(a)
    assert_equal(p.read(7), "one,two")

def test_putbackio_readline_from_putback_and_stream_short():
    """
    Test readline for a length shorter than the putback buffer
    """
    s = StringIO('one two\nthree four\n')
    s.seek(0)
    p = PutbackInput(s)
    a = p.read(4)
    b = p.read(4)
    p.putback(b)
    p.putback(a)
    assert_equal(p.readline(10), "one two\n")

def test_putbackio_readline_from_putback_and_stream_long():
    """
    Test readline for a length longer than the putback buffer
    """
    s = StringIO('one two\nthree four\n')
    s.seek(0)
    p = PutbackInput(s)
    a = p.read(2)
    b = p.read(2)
    p.putback(b)
    p.putback(a)
    assert_equal(p.readline(10), "one two\n")


def test_delimitedio_read():

    s = StringIO('one--two--three')
    s.seek(0)
    p = PutbackInput(s)
    assert_equal(DelimitedInput(p, '--').read(100), 'one')
    assert_equal(DelimitedInput(p, '--').read(100), 'two')
    assert_equal(DelimitedInput(p, '--').read(100), 'three')
    assert_equal(DelimitedInput(p, '--').read(100), '')

    # delimiters are handled correctly, even when they straddle a
    # block (as determined by ``size``)

    s = StringIO('one--two--three')
    s.seek(0)
    p = PutbackInput(s)
    d = DelimitedInput(p, '--')
    assert_equal(d.read(2), 'on')
    assert_equal(d.read(2), 'e')
    assert_equal(d.read(2), '')

def test_delimitedio_readline():

    s = StringIO('one\ntwo|\nthree\n')
    s.seek(0)
    p = PutbackInput(s)
    d = DelimitedInput(p, '|')
    assert_equal(d.readline(), 'one\n')
    assert_equal(d.readline(), 'two')
    assert_equal(d.readline(), '')

    # The case where a newline forms part of the delimiter
    s = StringIO('one\ntwo\n--three\n')
    s.seek(0)
    p = PutbackInput(s)
    d = DelimitedInput(p, '\n--')
    assert_equal(d.readline(), 'one\n')
    assert_equal(d.readline(), 'two')
    assert_equal(d.readline(), '')





# Copyright (c) 2007-2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

import pesto
from pesto.response import Response
from pesto.testing import TestApp
from pesto.caching import with_etag, etag_middleware

from nose.tools import assert_equal

def gen_etag(request):
    return "one"

def test_set_etag():

    @pesto.to_wsgi
    @with_etag(gen_etag)
    def app(request):
        return Response(['whoa nelly'])
    assert_equal(TestApp(app).get().get_header('ETag'), '"one"')

def test_set_etag_weak():

    @pesto.to_wsgi
    @with_etag(gen_etag, weak=True)
    def app(request):
        return Response(['whoa nelly'])

    assert_equal(TestApp(app).get().get_header('ETag'), 'W/"one"')

def test_if_none_match_hit():

    calls = []

    @pesto.to_wsgi
    @with_etag(gen_etag)
    def app(request):
        def gen_content():
            yield "whoa"
            yield " nelly"
            calls.append(1)
        return Response(gen_content())

    app = etag_middleware(app)

    result = TestApp(app).get(HTTP_IF_NONE_MATCH='"one"')
    assert_equal(result.get_header('ETag'), '"one"')
    assert_equal(result.status, "304 Not Modified")
    assert_equal(result.body, '')
    assert_equal(len(calls), 0)

def test_if_none_match_miss():

    calls = []

    @pesto.to_wsgi
    @with_etag(gen_etag)
    def app(request):
        def gen_content():
            yield "whoa"
            yield " nelly"
            calls.append(1)
        return Response(gen_content())

    app = etag_middleware(app)

    result = TestApp(app).get(HTTP_IF_NONE_MATCH='"two"')
    assert_equal(result.get_header('ETag'), '"one"')
    assert_equal(result.status, "200 OK")
    assert_equal(result.body, 'whoa nelly')
    assert_equal(len(calls), 1)




# Copyright (c) 2007-2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

from functools import wraps
import pesto
from pesto.response import Response
from pesto.request import Request
from pesto.testing import TestApp

from nose.tools import assert_equal

def test_spaces():

    def h1(request, value):
        return Response([value])

    d = pesto.dispatcher_app()
    d.match('/<value:unicode>', GET=h1)

    response = TestApp(d).get('/value%20with%20spaces')
    assert_equal(response.body, 'value with spaces')

def test_unbound_method():

    d = pesto.dispatcher_app()

    class Controller(object):
        @d.match('/h1', 'GET')
        def h1(request):
            return Response(["h1"])

        @d.match('/h2', 'GET')
        def h2(request):
            return Response(["h2"])

    d = TestApp(d)
    assert d.get('/h1').body == 'h1'
    assert d.get('/h2').body == 'h2'

def test_bound_method():

    d = pesto.dispatcher_app()

    class Greeter(object):

        def __init__(self, greeting):
            self.greeting = greeting

        def greet(self, request):
            return Response(self.greeting)

    g = Greeter('hello world!')
    d.match('/greeting', 'GET')(g.greet)

    d = TestApp(d)
    assert_equal(d.get('/greeting').body, 'hello world!')



def test_dispatcher_simple_match():

    # Ordinarily these would be pesto handler functions, but strings are easier for writing test cases
    handler1 = 'h1'
    handler2 = 'h2'

    dispatcher = pesto.dispatcher_app()
    dispatcher.match('/foo', GET=handler1)
    dispatcher.match('/bar', GET=handler2)

    request = Request(TestApp.make_environ(PATH_INFO='/foo'))
    assert_equal(
        list(dispatcher.gettarget('/foo', 'GET', request)),
        [(handler1, None, (), {})]
    )

    request = Request(TestApp.make_environ(PATH_INFO='/foo'))
    assert_equal(
        list(dispatcher.gettarget('/bar', 'GET', request)),
        [(handler2, None, (), {})]
    )

    request = Request(TestApp.make_environ(PATH_INFO='/baz'))
    assert_equal(
        list(dispatcher.gettarget('/baz', 'GET', request)),
        []
    )


def test_dispatcher_trailing_slashes():

    # Ordinarily these would be pesto handler functions, but strings are easier for writing test cases
    handler1 = 'h1'
    handler2 = 'h2'

    dispatcher = pesto.dispatcher_app()
    dispatcher.match('/foo', GET=handler1)

    request = Request(TestApp.make_environ(PATH_INFO='/foo'))
    assert_equal(
        list(dispatcher.gettarget('/foo', 'GET', request)),
        [(handler1, None, (), {})]
    )
    request = Request(TestApp.make_environ(PATH_INFO='/foo/'))
    assert_equal(
        list(dispatcher.gettarget('/foo/', 'GET', request)),
        [(handler1, None, (), {})]
    )
    request = Request(TestApp.make_environ(PATH_INFO='/foo////'))
    assert_equal(
        list(dispatcher.gettarget('/foo////', 'GET', request)),
        [(handler1, None, (), {})]
    )

def test_dispatcher_match_with_vars():

    handler1 = 'h1'
    handler2 = 'h2'

    dispatcher = pesto.dispatcher_app()
    dispatcher.match(r'/entries/new',        GET=handler2)
    dispatcher.match(r'/entries/<id:unicode>', GET=handler1)

    request = Request(TestApp.make_environ(PATH_INFO='/entries/new'))
    assert_equal(
        list(dispatcher.gettarget(request.path_info, request.request_method, request)),
        [(handler2, None, (), {}), (handler1, None, (), {'id' : 'new'})]
    )

    request = Request(TestApp.make_environ(PATH_INFO='/entries/blah'))
    assert_equal(
        list(dispatcher.gettarget(request.path_info, request.request_method, request)),
        [(handler1, None, (), {'id': 'blah'})]
    )

def test_combined_dispatcher():

        d1 = pesto.dispatcher_app()
        d1.match('/foo', GET=lambda request: Response(['d1:foo']))
        d2 = pesto.dispatcher_app()
        d2.match('/bar', GET=lambda request: Response(['d2:bar']))
        combined = pesto.dispatcher_app(debug=True).combine(d1, d2)

        d1 = TestApp(d1)
        d2 = TestApp(d2)
        combined = TestApp(combined)
        assert_equal(d1.get('/foo').body, 'd1:foo')
        assert_equal(combined.get('/foo').body, 'd1:foo')

        assert_equal(d2.get('/bar').body, 'd2:bar')
        assert_equal(combined.get('/bar').body, 'd2:bar')


def test_unicode():
    d = pesto.dispatcher_app()
    request = Request(TestApp.make_environ())
    d.match('/<:unicode>', GET='x'),
    assert_equal(list(d.gettarget('/xyzy', 'GET', request)), [('x', None, ('xyzy',), {})])
    assert_equal(list(d.gettarget('/xyzy/index.html', 'GET', request)), [])

def test_path():
    d = pesto.dispatcher_app()
    request = Request(TestApp.make_environ())
    d.match('/<:path>', GET='x'),
    assert_equal(list(d.gettarget('/xyzy', 'GET', request)), [('x', None, ('xyzy',), {})])
    assert_equal(list(d.gettarget('/xyzy/index.html', 'GET', request)), [('x', None, ('xyzy/index.html',), {})])

def test_any():
    d = pesto.dispatcher_app()
    request = Request(TestApp.make_environ())
    d.match('/<:any("about", "contact")>.html', GET='x'),
    assert_equal(list(d.gettarget('/xyzy', 'GET', request)), [])
    assert_equal(list(d.gettarget('/about.html', 'GET', request)), [('x', None, ('about',), {})])
    assert_equal(list(d.gettarget('/about.htm', 'GET', request)), [])
    assert_equal(list(d.gettarget('/contact.html', 'GET', request)), [('x', None, ('contact',), {})])
    assert_equal(list(d.gettarget('/xyzzy.html', 'GET', request)), [])


def test_predicate():

    def from_ip(addr):
        def from_ip(req):
            return req.remote_addr.startswith(addr)
        return from_ip

    d = pesto.dispatcher_app()

    @d.match('/', 'GET', predicate=from_ip('192.168.1.'))
    def internal(req):
        return Response(['internal'])

    @d.match('/', 'GET', predicate=from_ip('127.0.0.1'))
    def localhost(req):
        return Response(['localhost'])

    @d.match('/', 'GET')
    def external(req):
        return Response(['external'])

    d = TestApp(d)
    assert_equal(d.get('/', REMOTE_ADDR='127.0.0.1').body, 'localhost')
    assert_equal(d.get('/', REMOTE_ADDR='192.168.1.20').body, 'internal')
    assert_equal(d.get('/', REMOTE_ADDR='1.2.3.4').body, 'external')


def test_decorators():

    def to_json(func):
        """
        Wrap a pesto handler to return a JSON-encoded string from a python
        data structure.
        """
        try:
            import json
        except ImportError:
            import simplejson as json

        @wraps(func)
        def to_json(request, *args, **kwargs):
            result = func(request, *args, **kwargs)
            if isinstance(result, Response):
                return result
            return Response(
                content=[json.dumps(result)],
                content_type='application/json'
            )

        return to_json

    def to_text(func):
        """
        Wrap a pesto handler to return a comma separated string from a python list
        """
        @wraps(func)
        def to_text(request, *args, **kwargs):
            result = func(request, *args, **kwargs)
            if isinstance(result, Response):
                return result
            return Response(
                content=[','.join(result)],
                content_type='text/plain'
            )

        return to_text


    d = pesto.dispatcher_app()

    @d.match('/data.json', "GET", decorators=[to_json])
    @d.match('/data.text', "GET", decorators=[to_text])
    def getdata(request):
        return ['yo', 'ho', 'ho']

    request = Request(TestApp.make_environ())
    assert_equal(getdata(request), ['yo', 'ho', 'ho'])
    assert_equal(TestApp(d).get('/data.json').body, '["yo", "ho", "ho"]')
    assert_equal(TestApp(d).get('/data.text').body, 'yo,ho,ho')

def test_precedence():
    """
    Check that the first matching function is used
    """
    d = pesto.dispatcher_app()

    @d.match('/whuffle', 'GET')
    def whuffle(request):
        return Response(['whuffle-1'])

    @d.match('/whuffle', 'GET')
    def whuffle(request):
        return Response(['whuffle-2'])

    d = TestApp(d)
    assert_equal(d.get('/whuffle').body, 'whuffle-1')

def test_cache_with_predicate():

    predicate = lambda request: 'flobble' in request.form
    d = pesto.dispatcher_app(cache_size=10)

    @d.match('/whuffle', 'GET', predicate=predicate)
    def whuffle_with_flobble(request):
        return Response(['whuffle-flobble'])

    @d.match('/whuffle', 'GET')
    def whuffle_no_flobble(request):
        return Response(['whuffle'])

    d = TestApp(d)
    assert_equal(d.get('/whuffle').body, 'whuffle')
    assert_equal(d.get('/whuffle', data={'flobble': '1'}).body, 'whuffle-flobble')

    # Second pass to check that the cache does not store the wrong result
    assert_equal(d.get('/whuffle').body, 'whuffle')
    assert_equal(d.get('/whuffle', data={'flobble': '1'}).body, 'whuffle-flobble')

def test_url_generation():

    d = pesto.dispatcher_app()

    @d.match('/page/one', 'GET')
    def page_one(request):
        return Response([page_two.url()])

    @d.match('/page/two', 'GET')
    def page_two(request):
        return Response([page_one.url()])

    d = TestApp(d)
    assert_equal(d.get('/page/one').body, 'http://localhost/page/two')
    assert_equal(d.get('/page/two').body, 'http://localhost/page/one')

def test_url_generation_overriding_script_name():

    d = pesto.dispatcher_app()

    @d.match('/page/one', 'GET')
    def page_one(request):
        return Response([page_one.url(script_name='/buffalo')])

    d = TestApp(d)
    assert_equal(d.get('/page/one').body, 'http://localhost/buffalo/page/one')


def test_url_generation_overriding_netloc():

    d = pesto.dispatcher_app()

    @d.match('/page/one', 'GET')
    def page_one(request):
        return Response([page_one.url(netloc='example.org:8080')])

    d = TestApp(d)
    assert_equal(d.get('/page/one').body, 'http://example.org:8080/page/one')





# Copyright (c) 2007-2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

import threading

__docformat__ = 'restructuredtext en'
__version__ = '2'
__all__ = ['LRUCache']

class Node(object):
    """
    Node in a doubly-linked list
    """

    __slots__ = ['item', 'next', 'prev']

    def __init__(self, item, next, prev):
        self.item = item
        self.next = next
        self.prev = prev

    def __repr__(self):
        if self.next:
            next = self.next.item
        else:
            next = None
        if self.prev:
            prev = self.prev.item
        else:
            prev = None
        return repr((prev, self.item, next))


class LRUCache(dict):

    """
    Fixed size, least recently used (LRU) cache.

    This class implements (most) of the dict interface. When more items are
    added to the dict, than ``maxsize``, items are purged from the cache
    starting with the least recently used items.

    Usage of items is tracked through a doubly-linked list. As items are
    accessed they are moved to the front of the list. Items are purged from
    the end of the list.
    """

    def __init__(self, maxsize, purge_number=None, purge_listener=None):
        """
        Create a new Least Recently Used (LRU) caching object.

        maxsize
            the maximum number of items to store in this cache.

        purge_number
            the number of items to purge in one go.

        purge_listener
            a callable that will be invoked whenever items are
            purged from the cache, passing the list of (key,
            values) about to be deleted deleted. 
        """
        self.maxsize = maxsize
        self._first = None 
        self._last = None
        self.purge_listener = purge_listener
        if purge_number is None:
            purge_number = int(min(maxsize * 0.05, 100))
        self.purge_number = max(1, purge_number)
        self._lock = threading.Lock()

    def _moveToFront(self, node):
        """
        Move ``node`` to the front of the linked list
        """
        if self._first is node:
            return

        # Take node out of its list position
        try:
            node.prev.next = node.next
        except AttributeError: pass
        try:
            node.next.prev = node.prev
        except AttributeError: pass

        # If node is the last item, repoint _last
        if node is self._last:
            self._last = node.prev

        # And reinsert at the front
        try:
            self._first.prev = node
        except AttributeError: pass

        node.next = self._first
        node.prev = None
        self._first = node

    def __setitem__(self, key, value):
        self._lock.acquire()
        try:
            # Push node in at front of linked list
            try:
                oldval, node = super(LRUCache, self).__getitem__(key)
                self._moveToFront(node)
                super(LRUCache, self).__setitem__(key, (value, node))
            except KeyError:
                node = Node(key, self._first, None)
                if self._first:
                    self._first.prev = node

                if self._last is None:
                    self._last = node
                self._first = node
                super(LRUCache, self).__setitem__(key, (value, self._first))

            # Purge the cache if necessary
            if len(self) > self.maxsize:
                newlast = self._last
                if self.purge_listener is not None:
                    todel = []
                    for ix in xrange(self.purge_number):
                        todel.append((newlast.item, super(LRUCache, self).__getitem__(newlast.item)[0]))
                        super(LRUCache, self).__delitem__(newlast.item)
                        newlast = newlast.prev
                    self.purge_listener(todel)
                else:
                    for ix in xrange(self.purge_number):
                        super(LRUCache, self).__delitem__(newlast.item)
                        newlast = newlast.prev

                self._last = newlast
                if self._last:
                    self._last.next = None
        finally:
            self._lock.release()

    def __getitem__(self, key):
        self._lock.acquire()
        try:
            value, node = super(LRUCache, self).__getitem__(key)
            self._moveToFront(node)
            return value
        finally:
            self._lock.release()

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default

    def __delitem__(self, key):
        self._lock.acquire()
        try:
            value, node = super(LRUCache, self).__getitem__(key)
            # Take node out of its list position
            if node.prev is not None:
                node.prev.next = node.next
            if node.next is not None:
                node.next.prev = node.prev
            if node is self._first:
                self._first = node.next
            if node is self._last:
                self._last = node.prev
            del node
            return super(LRUCache, self).__delitem__(key)
        finally:
            self._lock.release()

    def tolist(self):
        l = []
        node = self._first
        while node is not None:
            l.append(node.item)
            node = node.next
        return l

    def items(self):
        return [ (key, value[0]) for key, value in super(LRUCache, self).items() ]

    def iterkeys(self):
        return self.d.iterkeys()

    def iteritems(self):
        for item in super(LRUCache, self).iteritems():
            yield item[0], item[1][0]

    def itervalues(self):
        for item in super(LRUCache, self).itervalues():
            yield item[0]

    def __repr__(self):
        l = []
        node = self._first
        while node:
            l.append(node.item)
            node = node.next
        return repr(l)

    def debug(self):
        l = []
        node = self._first
        while node:
            if node is self._first and node is self._last:
                l.append("FL%r" % node)
            elif node is self._first:
                l.append("F%r" % node)
            elif node is self._last:
                l.append("L%r" % node)
            else:
                l.append(repr(node))
            node = node.next
        return ", ".join(l)


def check_linkedlistintegrity():
    # Checks list for correct backwards links (prev) and 
    # circular references and that this is maintained as the cache is manipulated
    c = LRUCache(30)
    def test_integrity():
        node = c._first
        prev = None
        seenbefore = Set()
        while node is not None:
            seenbefore.add(node)
            self.assertTrue(node.prev is prev)
            prev = node
            self.assertTrue(node.next not in seenbefore)
            node = node.next

    for ix, ch in enumerate('abcdefghijklmnopqrstuvwxyz'):
        c[ch] = ix
        test_integrity()

    for position in xrange(26):
        node = c._first
        # Select the ``position``th element from the linked list...
        for iy in xrange(position):
            node = node.next

        # ...and get it out of the cache, forcing the cache to
        # reorder itself
        dummy = c[node.item]
        test_integrity()
        self.assertEqual(len(c.tolist()), 26)

